@@ -5,6 +5,8 @@
  */
 
 #include "utilities.h"
+#include "powerdevilwakeupprovider.h"
+#include "waittimerwakeupprovider.h"
 
 #include <QApplication>
 #include <QDBusReply>
@@ -40,6 +42,7 @@ Utilities::Utilities(QObject *parent)
                                      this))
     , m_hasPowerDevil(false)
     , m_timer(new QTimer(this))
+    , m_wakeupProvider(nullptr)
 {
     // TODO: It'd be nice to be able to have the daemon off if no alarms/timers are running.
     // However, with the current implementation, the client continuously thinks it is off.
@@ -49,10 +52,7 @@ Utilities::Utilities(QObject *parent)
     //         }
     //     });
 
-    // if PowerDevil is present, we can rely on PowerDevil to track time, otherwise we do it ourself
-    if (m_interface->isValid() && !noPowerDevil) {
-        m_hasPowerDevil = hasWakeup();
-    }
+    initWakeupProvider();
 
     bool success = QDBusConnection::sessionBus().registerObject(QStringLiteral("/Utility"),
                                                                 QStringLiteral("org.kde.PowerManagement"),
@@ -60,13 +60,6 @@ Utilities::Utilities(QObject *parent)
                                                                 QDBusConnection::ExportScriptableSlots);
     qDebug() << "Registered on DBus:" << success;
 
-    if (hasPowerDevil()) {
-        qDebug() << "PowerDevil found, using it for time tracking.";
-    } else {
-        initWorker();
-        qDebug() << "PowerDevil not found, using wait worker thread for time tracking.";
-    }
-
     if (!noPowerDevil) { // do not watch for powerdevil if we explicitly turned it off
         auto m_watcher = new QDBusServiceWatcher{POWERDEVIL_SERVICE_NAME,
                                                  QDBusConnection::sessionBus(),
@@ -74,14 +67,20 @@ Utilities::Utilities(QObject *parent)
         connect(m_watcher, &QDBusServiceWatcher::serviceRegistered, this, [this]() {
             qDebug() << "PowerDevil found on DBus";
             m_hasPowerDevil = hasWakeup();
-            if (m_hasPowerDevil && m_timerThread)
-                m_timerThread->quit();
+            if (m_hasPowerDevil) {
+                m_wakeupProvider->deleteLater();
+                m_wakeupProvider = new PowerDevilWakeupProvider(this);
+                connect(m_wakeupProvider, &AbstractWakeupProvider::wakeup, this, &Utilities::wakeup);
+            }
 
             Q_EMIT needsReschedule();
         });
         connect(m_watcher, &QDBusServiceWatcher::serviceUnregistered, this, [this]() {
+            qDebug() << "PowerDevil left DBus";
             m_hasPowerDevil = false;
-            initWorker();
+            m_wakeupProvider->deleteLater();
+            m_wakeupProvider = new WaitTimerWakeupProvider(this);
+            connect(m_wakeupProvider, &AbstractWakeupProvider::wakeup, this, &Utilities::wakeup);
 
             Q_EMIT needsReschedule();
         });
@@ -91,6 +90,23 @@ Utilities::Utilities(QObject *parent)
     m_timer->start(60 * 1000);
 }
 
+void Utilities::initWakeupProvider()
+{
+    // if PowerDevil is present, we can rely on PowerDevil to track time, otherwise we do it ourself
+    if (m_interface->isValid() && !noPowerDevil) {
+        m_hasPowerDevil = hasWakeup();
+    }
+
+    if (hasPowerDevil()) {
+        m_wakeupProvider = new PowerDevilWakeupProvider(this);
+        qDebug() << "PowerDevil found, using it for time tracking.";
+    } else {
+        m_wakeupProvider = new WaitTimerWakeupProvider(this);
+        qDebug() << "PowerDevil not found, using wait worker thread for time tracking.";
+    }
+    connect(m_wakeupProvider, &AbstractWakeupProvider::wakeup, this, &Utilities::wakeup);
+}
+
 bool Utilities::hasPowerDevil()
 {
     return m_hasPowerDevil;
@@ -98,87 +114,12 @@ bool Utilities::hasPowerDevil()
 
 int Utilities::scheduleWakeup(quint64 timestamp)
 {
-    if (hasPowerDevil()) {
-        QDBusReply<uint> reply = m_interface->call(QStringLiteral("scheduleWakeup"), QStringLiteral("org.kde.kclockd"), QDBusObjectPath("/Utility"), timestamp);
-        if (reply.isValid()) {
-            m_powerDevilCookies.append(reply.value());
-        } else {
-            qDebug() << "Invalid reply, error: " << reply.error();
-        }
-        return reply.value();
-    } else {
-        m_waitWorkerCookies.append({++m_cookie, timestamp});
-        schedule();
-        return m_cookie;
-    }
+    return m_wakeupProvider->scheduleWakeup(timestamp);
 }
 
 void Utilities::clearWakeup(int cookie)
 {
-    if (hasPowerDevil()) {
-        auto index = m_powerDevilCookies.indexOf(cookie);
-        if (index != -1) {
-            m_interface->call(QStringLiteral("clearWakeup"), cookie);
-            m_powerDevilCookies.removeAt(index);
-        }
-    } else {
-        for (auto index = m_waitWorkerCookies.begin(); index < m_waitWorkerCookies.end(); ++index) {
-            int pairCookie = (*index).first;
-            if (cookie == pairCookie) {
-                m_waitWorkerCookies.erase(index);
-            }
-        }
-
-        // ensure that we schedule the next queued wakeup
-        schedule();
-    }
-}
-
-void Utilities::wakeupCallback(int cookie)
-{
-    qDebug() << "Received wakeup callback.";
-    auto index = m_powerDevilCookies.indexOf(cookie);
-
-    if (index == -1) {
-        // something must be wrong here, return and do nothing
-        qDebug() << "Callback ignored (wrong cookie).";
-    } else {
-        // remove token
-        m_powerDevilCookies.removeAt(index);
-        Q_EMIT wakeup(cookie);
-    }
-}
-
-void Utilities::schedule()
-{
-    auto eternity = std::numeric_limits<unsigned long long>::max();
-    auto minTime = eternity;
-
-    for (auto tuple : m_waitWorkerCookies) {
-        if (minTime > tuple.second) {
-            minTime = tuple.second;
-            m_currentCookie = tuple.first;
-        }
-    }
-
-    if (minTime != eternity) { // only schedule worker if we have something to wait on
-        m_worker->setNewTime(minTime); // Unix uses int64 internally for time
-    }
-}
-
-void Utilities::initWorker()
-{
-    if (!m_timerThread) {
-        m_timerThread = new QThread(this);
-        m_worker = new AlarmWaitWorker();
-        m_worker->moveToThread(m_timerThread);
-        connect(m_worker, &AlarmWaitWorker::finished, this, [this] {
-            // notify time is up
-            Q_EMIT wakeup(m_currentCookie);
-            clearWakeup(m_currentCookie);
-        });
-    }
-    m_timerThread->start();
+    m_wakeupProvider->clearWakeup(cookie);
 }
 
 bool Utilities::hasWakeup()
@@ -217,6 +158,16 @@ void Utilities::wakeupNow()
     QDBusConnection::sessionBus().call(wakeupCall);
 }
 
+void Utilities::wakeupCallback(int cookie)
+{
+    if (m_wakeupProvider) {
+        qDebug() << "Utilities::wakeupCallback called with cookie:" << cookie;
+        static_cast<PowerDevilWakeupProvider *>(m_wakeupProvider)->wakeupCallback(cookie);
+    } else {
+        qWarning() << "Wakeup provider not initialized";
+    }
+}
+
 // hack, use timer count to keep alive
 void Utilities::keepAlive()
 {
